import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbGlobalMap;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;

public class XC10SaveToCache extends MbJavaComputeNode {

	private String responseQueue;

	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		// Define the out terminal
		MbOutputTerminal out = getOutputTerminal("out");
		
		// Define the message assemblies
		MbMessage inMessage = inAssembly.getMessage();
		MbMessage outMessage = new MbMessage(inMessage);
		MbMessageAssembly outAssembly = new MbMessageAssembly(inAssembly,outMessage);

		// Get the original MsgId and ReplyToQ of the incoming request
		MbElement rootEl        = outAssembly.getMessage().getRootElement();
		MbElement replyToQEl    = rootEl.getFirstElementByPath("/MQMD/ReplyToQ");
		String    replyToQ      = replyToQEl.getValueAsString();
		
		MbElement msgIdEl       = rootEl.getFirstElementByPath("/MQMD/MsgId");
		String    msgId         = msgIdEl.getValueAsString();
		MbElement sourceQueueEl = rootEl.getFirstElementByPath("/MQMD/SourceQueue");
		String    sourceQueue   = sourceQueueEl.getValueAsString(); 
		/*
		 * We're about to overwrite the original ReplyToQ of this message.
		 * So, write it to the XC10 cache for safe keeping. This will be 
		 * used subsequently by the XC10 Response message flow to send the 
		 * response back to the original reply queue.
		 */
		System.out.println("Hello  [");

		/* TH?? New code to put info in XC10 external map */
		
		
		MbGlobalMap xc10Map = MbGlobalMap.getGlobalMap("userXXdatagrid","CS_XC10DATAGRID");
		
		xc10Map.put(msgId, replyToQ);
		System.out.println("Data to put in XC11 map [" + xc10Map + "] using key [" + msgId + "]: " + replyToQ);

		if (getUserDefinedAttribute("ResponseQueue").toString()=="")
		{System.out.println("User def attribute empty ! ");} else 
			System.out.println("User Def OK ! ");
		
		
		// Figure out which instance we're in, by looking at original source queue
		String instance = "";
		for(int i = sourceQueue.length() - 1; i >= 0; i--) {
			String c = sourceQueue.substring(i, i+1);

			// not a digit? Done
			if(
				   !"0".equals(c)
				&& !"1".equals(c)
				&& !"2".equals(c)
				&& !"3".equals(c)
				&& !"4".equals(c)
				&& !"5".equals(c)
				&& !"6".equals(c)
				&& !"7".equals(c)
				&& !"8".equals(c)
				&& !"9".equals(c)
			) {
				break;
			}
			instance = c + instance;
		}
				
		// Now we set ReplyToQ to an appropriate value so that the subsequent retrieveCustomer flow will send the output to 
		// the right destination.
		// The ReplyToQ is actually going to be set based on the value of responseQueue, which
		// is a user-defined property of the Request Flow. 
		// There are two cases:
		// 1. The Response flow is located in the same broker as the Request (different EG)
		//     In this case, the retrieveCustomer flow puts the output on GLOBAL.CACHE.RESPONSE.IN, which is a local queue.
		// 2. The Response flow is located in a separate broker (BROKER2, using QMGR2). 
		//     In this case, the retrieveCustomer flow puts the output message on GLOBAL.CACHE.RESPONSE.REMOTE, which is
		//     a remote queue pointing to GLOBAL.CACHE.RESPONSE.IN in QMGR2. It can therefore be processed by a copy of the
		//     same message flow running in BROKER2.
		
		if (getUserDefinedAttribute("ResponseQueue").toString()=="")
		{System.out.println("User def attribute empty ! ");} else 
			System.out.println("User Def OK ! ");


		
		responseQueue = getUserDefinedAttribute("ResponseQueue").toString();
		outMessage.getRootElement().getFirstElementByPath("/MQMD/ReplyToQ").setValue(responseQueue + instance);
		
		// Go!
		out.propagate(outAssembly);
		
		return;
	}

}
